import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Chart } from '@antv/g2'
import DataSet from '@antv/data-set'

import ChartDrawerForm from '@/templates/zshare/chartcompile'
import './index.scss'

class LineChart extends Component {
  static propTpyes = {
    type: PropTypes.string,
    dict: PropTypes.object,
    plot: PropTypes.object,
    columns: PropTypes.array,
    plotchange: PropTypes.func
  }

  state = {
    visible: true
  }

  componentDidMount () {
    this.viewrender()
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    if (!is(fromJS(this.props.plot), fromJS(nextProps.plot))) {
      this.setState({}, () => {
        this.viewrender()
      })
    }
  }

  getdata = (X_axis, Y_axis) => {
    const { type } = this.props
    let data = []
    let xdata = ['周一', '周二', '周三', '周四', '周五', '周六', '周日']
    let point = 7

    if (type === 'pie') {
      xdata = ['事例一', '事例二', '事例三', '事例四', '事例五']
      point = 5
    }

    for (let i = 0; i < point; i++) {
      let item = {}

      item[X_axis] = xdata[i]

      if (typeof(Y_axis) === 'string') {
        item[Y_axis] = Math.floor(Math.random() * 5 * (i + 1)) + i
      } else {
        Y_axis.forEach(y => {
          item[y] = Math.floor(Math.random() * 5 * (i + 1)) + i
        })
      }

      data.push(item)
    }

    return data
  }

  viewrender = () => {
    const { type } = this.props

    if (type === 'line') {
      this.linerender()
    } else if (type === 'bar') {
      this.barrender()
    } else if (type === 'pie') {
      this.pierender()
    }
  }

  linerender = () => {
    const { plot, columns } = this.props

    let transfield = {}
    columns.forEach(col => {
      if (col.field) {
        transfield[col.field] = col.label
      }
    })
    // const colors = ['#f49d37', '#f03838', '#35d1d1', '#5be56b', '#4e7af0', '#ebcc21']
    let X_axis = plot.Xaxis || 'x'
    let Y_axis = plot.Yaxis || ['y']

    let data = this.getdata(X_axis, Y_axis)

    const ds = new DataSet()
    const dv = ds.createView().source(data)

    dv.transform({
      type: 'fold',
      fields: [...Y_axis],
      key: 'key', // key字段
      value: 'value', // value字段
      // retains: [], // 保留字段集，默认为除 fields 以外的所有字段
    })

    if (plot.Xaxis) {
      dv.transform({
        type: 'map',
        callback(row) {
          row.key = transfield[row.key]
          return row
        },
      })
    }
    
    const chart = new Chart({
      container: plot.uuid,
      autoFit: true,
      height: plot.height || 400
    })

    chart.data(dv.rows)

    if (plot.coordinate !== 'polar') {
      chart.scale(X_axis, {
        range: [0, 1]
      })
    }
    chart.scale('value', {
      nice: true
    })

    if (!plot.legend || plot.legend === 'hidden') {
      chart.legend(false)
    } else {
      chart.legend({
        position: plot.legend
      })
    }

    if (plot.tooltip !== 'true') {
      chart.tooltip(false)
    } else {
      chart.tooltip({
        shared: true
      })
    }

    if (plot.transpose === 'true') {
      chart.coordinate().transpose()
    }

    if (plot.coordinate === 'polar') {
      chart.coordinate('polar', {
        innerRadius: 0.1,
        radius: 0.8
      })
    }

    let _chart = chart
      .line()
      .position(`${X_axis}*value`)
      .color('key')
      .shape(plot.shape || 'smooth')

    if (plot.label === 'true') {
      _chart.label('value')
    }

    if (plot.point === 'true') {
      chart
        .point()
        .position(`${X_axis}*value`)
        .color('key')
        .size(3)
        .shape('circle')
    }
    
    // chart.interaction('element-active') 
    // chart.removeInteraction('legend-filter') // 自定义图例，移除默认的分类图例筛选交互
    chart.render()
  }

  barrender = () => {
    const { plot, columns } = this.props

    let transfield = {}
    columns.forEach(col => {
      if (col.field) {
        transfield[col.field] = col.label
      }
    })
    let X_axis = plot.Xaxis || 'x'
    let Y_axis = plot.Yaxis || ['y']
    if (!plot.Yaxis && plot.modelId !== 'bar1') {
      Y_axis = ['bar1', 'bar2']
    }

    let data = this.getdata(X_axis, Y_axis)

    const ds = new DataSet()
    const dv = ds.createView().source(data)

    dv.transform({
      type: 'fold',
      fields: [...Y_axis],
      key: 'key',
      value: 'value'
    })

    if (plot.Xaxis) {
      dv.transform({
        type: 'map',
        callback(row) {
          row.key = transfield[row.key]
          return row
        },
      })
    }
    
    const chart = new Chart({
      container: plot.uuid,
      autoFit: true,
      height: plot.height || 400
    })

    chart.data(dv.rows)

    chart.scale('value', {
      nice: true
    })

    if (!plot.legend || plot.legend === 'hidden') {
      chart.legend(false)
    } else {
      chart.legend({
        position: plot.legend
      })
    }

    if (plot.tooltip !== 'true') {
      chart.tooltip(false)
    } else {
      chart.tooltip({
        shared: true
      })
    }

    if (plot.transpose === 'true') {
      chart.coordinate().transpose()
    }

    if (plot.coordinate === 'polar') {
      chart.coordinate('polar', {
        innerRadius: 0.1,
        radius: 0.8
      })
    }

    if (plot.adjust !== 'stack') {
      chart
        .interval()
        .position(`${X_axis}*value`)
        .color('key')
        .adjust([
          {
            type: 'dodge',
            marginRatio: 0
          }
        ])
        .shape(plot.shape || 'rect')
    } else if (plot.adjust === 'stack') {
      chart
        .interval()
        .position(`${X_axis}*value`)
        .color('key')
        .adjust('stack')
        .shape(plot.shape || 'rect')
    }

    chart.render()
  }

  pierender = () => {
    const { plot, columns } = this.props

    let transfield = {}
    columns.forEach(col => {
      if (col.field) {
        transfield[col.field] = col.label
      }
    })
    let X_axis = plot.Xaxis || 'x'
    let Y_axis = plot.Yaxis || 'y'

    let data = this.getdata(X_axis, Y_axis)

    const ds = new DataSet()
    const dv = ds.createView().source(data)

    if (plot.pieshow !== 'value') {
      dv.transform({
        type: 'percent',
        field: Y_axis,
        dimension: X_axis,
        as: 'percent'
      })
    }
    
    const chart = new Chart({
      container: plot.uuid,
      autoFit: true,
      height: plot.height || 400
    })

    chart.data(dv.rows)

    if (plot.pieshow !== 'value') {
      chart.scale('percent', {
        formatter: (val) => {
          val = val * 100 + '%'
          return val
        }
      })
    }

    chart.coordinate('theta', {
      innerRadius: plot.shape === 'ring' ? 0.6 : 0,
      radius: 0.75,
    })

    if (!plot.legend || plot.legend === 'hidden') {
      chart.legend(false)
    } else {
      chart.legend({
        position: plot.legend
      })
    }

    if (plot.tooltip !== 'true') {
      chart.tooltip(false)
    } else {
      chart.tooltip({
        showTitle: false,
        showMarkers: false
      })
    }

    if (plot.pieshow !== 'value') {
      let _chart = chart
        .interval()
        .adjust('stack')
        .position('percent')
        .color(X_axis)
        .tooltip(X_axis + '*percent', (item, percent) => {
          percent = (percent * 100).toFixed(2) + '%'
          return {
            name: item,
            value: percent
          }
        })

      if (plot.label === 'true') {
        let setting = {
          content: (data) => {
            return `${data[X_axis]}: ${(data.percent * 100).toFixed(2)}%`
          }
        }

        if (plot.labelLayout === 'overlap') {
          setting.type = 'pie'
          setting.layout = {
            type: 'overlap'
          }
          setting.offset = 0
          // setting.style = {
          //   textAlign: 'center',
          //   fontSize: 12,
          //   fill: '#535353'
          // }
        }

        _chart.label('percent', setting)
      }
      
    } else {
      let _chart = chart
        .interval()
        .adjust('stack')
        .position(Y_axis)
        .color(X_axis)
        .tooltip(X_axis + '*' + Y_axis, (item, value) => {
          return {
            name: item,
            value: value
          }
        })

      if (plot.label === 'true') {
        let setting = {
          content: (data) => {
            return `${data[X_axis]}: ${data[Y_axis]}`
          }
        }

        if (plot.labelLayout === 'overlap') {
          setting.type = 'pie'
          setting.layout = {
            type: 'overlap'
          }
          setting.offset = 0
        }

        _chart.label(Y_axis, setting)
      }
    }
    
    chart.render()
  }

  plotChange = (values) => {
    const { plot } = this.props

    this.props.plotchange({...plot, ...values})
  }

  render() {
    const { plot, type } = this.props

    return (
      <div className="line-chart-edit-box" style={{minHeight: plot.height ? plot.height + 50 : 450}}>
        <p className="chart-title">{plot.title}</p>
        <ChartDrawerForm type={type} columns={this.props.columns} dict={this.props.dict} plot={plot} plotchange={this.plotChange} />
        <div className="canvas" id={plot.uuid}></div>
      </div>
    )
  }
}

export default LineChart